@@ -6,10 +6,10 @@ from trytond.pyson import Eval, Bool
 from trytond.pool import Pool, PoolMeta
 
 __all__ = ['Configuration', 'Sale', 'SaleLine']
-__metaclass__ = PoolMeta
 
 
 class Configuration:
+    __metaclass__ = PoolMeta
     __name__ = 'sale.configuration'
     sale_carrier = fields.Property(fields.Many2One('carrier', 'Carrier',
         domain=[('carrier_product.salable', '=', True)],
@@ -28,6 +28,7 @@ class Configuration:
 
 
 class Sale:
+    __metaclass__ = PoolMeta
     __name__ = 'sale.sale'
     carrier = fields.Many2One('carrier', 'Carrier',
         domain=[('carrier_product.salable', '=', True)],
@@ -143,23 +144,13 @@ class Sale:
                         })
         return shipments
 
-    def _get_invoice_line_sale_line(self, invoice_type):
-        result = super(Sale, self)._get_invoice_line_sale_line(invoice_type)
-        if self.shipment_cost_method == 'shipment':
-            for line in self.lines:
-                if line.id in result and line.shipment_cost:
-                    del result[line.id]
-        return result
-
-    def create_invoice(self, invoice_type):
+    def create_invoice(self):
         pool = Pool()
         Invoice = pool.get('account.invoice')
         Shipment = pool.get('stock.shipment.out')
 
-        invoice = super(Sale, self).create_invoice(invoice_type)
-        if (invoice
-                and invoice_type == 'out_invoice'
-                and self.shipment_cost_method == 'shipment'):
+        invoice = super(Sale, self).create_invoice()
+        if invoice and self.shipment_cost_method == 'shipment':
             for shipment in self.shipments:
                 if (shipment.state == 'done'
                         and shipment.carrier
@@ -178,6 +169,14 @@ class Sale:
 
 
 class SaleLine:
+    __metaclass__ = PoolMeta
     __name__ = 'sale.line'
     shipment_cost = fields.Numeric('Shipment Cost',
         digits=(16, Eval('_parent_sale', {}).get('currency_digits', 2)))
+
+    def _get_invoice_line_quantity(self):
+        quantity = super(SaleLine, self)._get_invoice_line_quantity()
+        if (self.shipment_cost
+                and self.sale.shipment_cost_method == 'shipment'):
+            return 0
+        return quantity
